/*
 * ShortCutManager
 * Version 2
 * Created by Rudolf No
 * 23.12.2005
 *
 * Licence Statement
 * Version:  MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1  (the "License"); you may  not use this  file except in
 * compliance with the License.  You  may obtain a copy of the License
 * at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the  License  for  the   specific  language  governing  rights  and
 * limitations under the License.
 */

//Global Objects
var ShortCutManager2_allInstances = new Array();
var ShortCutManager2_eventListenerAlreadyAdded = false;
var ShortCutManager2_eventListenerForElementAlreadyAdded = false;

function ShortCutManager2(){
    this.shortCuts = new Object();
    ShortCutManager2_allInstances[length] = this;
}

function ShortCutManager2_getInstance(){
	if(!ShortCutManager2_eventListenerAlreadyAdded){
		window.addEventListener("keydown", ShortCutManager2_globalOnEvent, true);	
		ShortCutManager2_eventListenerAlreadyAdded = true;
	}
	return new ShortCutManager2();
}
ShortCutManager2.getInstance=ShortCutManager2_getInstance;

function ShortCutManager2_addJsShortCut(keyCode, modifierMask, jsCode){
    if(modifierMask==null){
        modifierMask = 0;
    }
    var shortCutKey = ShortCutManager2_createShortCutKey(keyCode, modifierMask)
    this.shortCuts[shortCutKey] = new ShortCut2(jsCode);
}
ShortCutManager2.prototype.addJsShortCut = ShortCutManager2_addJsShortCut;

/*
 * param: combinedKeyCode = keyCode << 4 | Event.ALT_MASK | Event.CONTROL_MASK | Event.SHIFT_MASK
 */
function ShortCutManager2_addJsShortCutWithCombinedKeyCode(combinedKeyCode, jsCode){
    var shortcutArray = this.shortCuts[combinedKeyCode];
    var newShortCut = new ShortCut2(jsCode);
    if(shortcutArray==null)
        this.shortCuts[combinedKeyCode] = new Array(newShortCut);
    else
        shortcutArray[shortcutArray.length] = newShortCut;
}
ShortCutManager2.prototype.addJsShortCutWithCombinedKeyCode = ShortCutManager2_addJsShortCutWithCombinedKeyCode

/*
 * param: combinedKeyCode = keyCode << 4 | Event.ALT_MASK | Event.CONTROL_MASK | Event.SHIFT_MASK
 */
function ShortCutManager2_addJsShortCutForElement(elementId, keyCode, modifierMask, jsCode){
	var element = document.getElementById(elementId);
	if(!element)
		return;
    var shortCutKey = ShortCutManager2_createShortCutKey(keyCode, modifierMask, elementId)
	this.addJsShortCutWithCombinedKeyCode(shortCutKey, jsCode)
	if(!ShortCutManager2_eventListenerForElementAlreadyAdded){
		window.addEventListener("keydown", ShortCutManager2_globalOnElementEvent, true);
		ShortCutManager2_eventListenerForElementAlreadyAdded = true;
	}
}
ShortCutManager2.prototype.addJsShortCutForElement = ShortCutManager2_addJsShortCutForElement

function ShortCutManager2_globalOnEvent(event, elementId){
	for(var i=0; i<ShortCutManager2_allInstances.length; i++){
		var scm = ShortCutManager2_allInstances[i];
		scm.onEvent(event, elementId);
	}
}

function ShortCutManager2_globalOnElementEvent(event){
	var srcElement = event.originalTarget;
	if(srcElement.id)
		ShortCutManager2_globalOnEvent(event, srcElement.id)
	else 
		ShortCutManager2_globalOnEvent(event, srcElement.name);
}

function ShortCutManager2_onEvent(event, elementId){
	var shortCuts = this.shortCuts;
    var shortCutKey = ShortCutManager2_encodeEvent(event);
    if(elementId)
    	shortCutKey = elementId + "_" + shortCutKey;
    var shortCutArray = shortCuts[shortCutKey]
    if(shortCutArray){
        this.currentEvent = event;
        for(var i=0; i<shortCutArray.length; i++){
            shortCutArray[i].onEvent(event);
        }
    }else{
        this.currentEvent = null;
    }
}
ShortCutManager2.prototype.onEvent = ShortCutManager2_onEvent;

function ShortCutManager2_clearShortCuts(combinedKeyCode){
    this.shortCuts = new Object();
}
ShortCutManager2.prototype.clearShortCuts=ShortCutManager2_clearShortCuts;

function ShortCutManager2_createShortCutKey(keyCode, modifierMask, elementId){
	var shortCutKey = keyCode << 4 | modifierMask;
	if(elementId)
		shortCutKey = elementId + "_" + shortCutKey;
	return shortCutKey;
}
/*
 * Encodes KeyEvent
 */
function ShortCutManager2_encodeEvent(event){
    return event.keyCode << 4 | ShortCutManager2_encodeEventModifier(event);
}

function ShortCutManager2_encodeEventModifier(event){
    return event.altKey * Event.ALT_MASK |
        event.ctrlKey * Event.CONTROL_MASK |
        event.shiftKey * Event.SHIFT_MASK |
        event.metaKey * Event.META_MASK;
}
ShortCutManager2.encodeEventModifier=ShortCutManager2_encodeEventModifier;

/*
 * Construktor fr Shortcut
 */
function ShortCut2(jsCode, clientId){
    this.jsCode = jsCode.replace(/'/g, '"');
    this.clientId = clientId;
}

function ShortCut2_onEvent(event){
    try{
        window.eval(this.jsCode);
    }catch(e){}
    event.preventDefault();
    event.stopPropagation();
}
ShortCut2.prototype.onEvent = ShortCut2_onEvent;

//Constants
ShortCutManager2.NONE = 0;
ShortCutManager2.ALT = Event.ALT_MASK;
ShortCutManager2.CTRL = Event.CONTROL_MASK;
ShortCutManager2.SHIFT = Event.SHIFT_MASK;
ShortCutManager2.CTRL_SHIFT = Event.CONTROL_MASK | Event.SHIFT_MASK;
ShortCutManager2.ALT_SHIFT = Event.ALT_MASK | Event.SHIFT_MASK;